
#include <stdio.h>
#include <stdlib.h>


#define TYPE long double


/**
 * Print the usage of this program
 */
void printUsage(int argc, char** argv)
{
	printf("Usage: %s p1x p1y p2x p2y px py eps num\n", argv[0]);
	printf("\n");
	printf("Calculates oriented triangle sized resp. their sign which have fixed\n");
	printf("points at (p1x, p1y) and (p2x, p2y). The third point varies in a\n");
	printf("surrounding square - in both directions num-many steps. \n");
}


/**
 * Calculate oriented size of triangle (p1x,p1y)--(p2x,p2y)--(ppx, ppy)
 *
 *             p1x, p1y, 1
 * return det( p2x, p2y, 1 )
 * 			   ppx, ppy, 1 
 * */
TYPE triangleSize(TYPE p1x, TYPE p1y, TYPE p2x, TYPE p2y,
				TYPE ppx, TYPE ppy)
{
	TYPE a = p1x*p2y;
	TYPE b = p2x*ppy;
	TYPE c = ppx*p1y;
	TYPE d = p1x*ppy;
	TYPE e = p2x*p1y;
	TYPE f = ppx*p2y;


	//In the following are at least two alternatives...

	//Calc oriented size of triangle
	//TYPE triangle = a - d + b - e + c -f;
	
	//With this ordering the symmetric property does NOT hold
	//(as good as in the version above)
	TYPE triangle = a + b + c - d - e - f;

	return triangle;
}


/**
 * Determine the smallest value delta where 1.0 != 1.0+delta
 */
TYPE getDelta()
{
	const TYPE ref = 1.0;
	TYPE delta = 1.0;

	//Reduce 1.0, until there is no difference between 1.0 and 1.0+delta
	while( 1 )
	{
			//Compare values
			//We need them, otherwise the intermediate values are in
			//long double precision --> 64bit mantissa!
			TYPE cmp = delta/2.0;
			cmp += ref;

			//If the values are equal --> return current delta
			if( ref == cmp )
				return delta;

			//Otherwise do next step
			delta /= 2.0;
	}

	return delta;
}


/**
 * Print the sign (+, 0, -) of val.
 */
void printSign( TYPE val)
{
	if( val==0 )
		printf("0");

	else if( val>0 )
		printf("+");

	else
		printf("-");
}


int main(int argc, char** argv)
{
	if( argc != 8 )
	{
		printUsage( argc, argv );
		return -1;
	}

	const TYPE p1x = atof(argv[1]);
	const TYPE p1y = atof(argv[2]);
	const TYPE p2x = atof(argv[3]);
	const TYPE p2y = atof(argv[4]);
	const TYPE px = atof(argv[5]);
	const TYPE py = atof(argv[6]);
	const int num = atoi(argv[7]);

	TYPE delta = getDelta();

	printf("Run test with\n");
	printf("\tp1x = %Lf\n", p1x);
	printf("\tp1y = %Lf\n", p1y);
	printf("\tp2x = %Lf\n", p2x);
	printf("\tp2y = %Lf\n", p2y);
	printf("\tpx = %Lf\n", px);
	printf("\tpy = %Lf\n", py);
	printf("\tnum = %d\n", num);
	printf("\tdelta = %.20Lf\n", delta);


	TYPE y = py - (num/2)*delta;

	for(int iy=0; iy<num; iy++, y+=delta)
	{
		TYPE x = px - (num/2)*delta;

		for(int ix=0; ix<num; ix++, x+=delta)
		{
			printSign( triangleSize(p1x, p1y, p2x, p2y, x, y) );
		}

		printf("\n");
	}

	return 0;
}




